#!/usr/bin/perl

use Test::More tests => 12;

use FindBin qw($Bin);
use lib '/usr/local/nagios/perl/lib';
use lib "$Bin/lib",              "$Bin/../lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";
use strict;

use Data::Dump qw(dump);
use File::Next;
use File::Basename;

use Test::MockObject;

use Catalyst::Request;
use Test::MockObject::Extends;

my $req = Catalyst::Request->new();
$req = Test::MockObject::Extends->new($req);
my $cookies = {};
$req->mock( "cookies", sub {$cookies} );
my $req_params = {};
$req->mock( "params", sub {$req_params} );
my $log = Test::MockObject->new();
my $log_debug;
my $log_info;
my $log_error;
$log->mock( "debug", sub { shift; $log_debug = shift } );
$log->mock( "info",  sub { shift; $log_info  = shift } );
$log->mock( "error", sub { shift; $log_error = shift } );

my $c     = Test::MockObject->new;
my $stash = {};
my $served_filename;

#my $log_error;
my $loc_message;
my $forward;

#my $config = {};
my $user     = Test::MockObject->new();
my $username = "robin";
$user->mock( "username", sub {$username} );
$c->mock(
    "stash",
    sub {
        shift;
        if (@_) { $stash->{ $_[0] } = $_[1] }
        else    {$stash}
    }
);
$c->mock( "serve_static_file", sub { shift; $served_filename = shift } );
$c->mock( "log", sub {$log} );
$c->mock( "loc", sub { shift; $loc_message = join( ":", @_ ) } );
$c->mock( "request", sub {$req} );
$c->mock( "req",     sub {$req} );
$c->mock( "debug",   sub {0} );
$c->mock( "forward", sub { shift; $forward = shift } );
$c->mock( "user",    sub {$user} );

my $public_rrdgraphs = 1;
my $role_access      = Test::MockObject->new;
$c->mock( "public_role", sub {$role_access} );
$role_access->mock( "has_access", sub {$public_rrdgraphs} );

use_ok("Opsview::Web::Controller::Common");

# set up rrd directory - files are stored in xml format and restored into
# rrd data file format so that tests will work on amd64 systems too
my $file_filter = sub { $_ && $_ =~ m/\.xml$/ };
my $rrd_xml = File::Next::files( { file_filter => $file_filter, }, "$Bin/var/rrd" );
while ( defined( my $dump = $rrd_xml->() ) ) {
    my ( $filename, $dir, $suffix ) = fileparse( $dump, qr/\.xml$/ );
    if ( -s $dump ) {
        system("rrdtool restore -f $dump $dir/$filename.rrd") == 0 or die 'rrdtool restore failure: ', $!;
    }
    else {
        system("touch $dir/$filename.rrd") == 0 or die 'touch failure: ', $!;
    }
}

$Opsview::Web::Controller::Common::rrddir = "$Bin/var/rrd";

my $rrds     = Opsview::Web::Controller::Common->list_rrds($c);
my $expected = {
    errors       => [],
    hosts        => {},
    services     => {},
    metrics      => {},
    metric_order => [],
    summary      => {
        host_services        => 0,
        hosts                => 0,
        services             => 0,
        metrics              => 0,
        host_service_metrics => 0,
        uoms                 => 0,
    },
    uoms => {},
};

is_deeply( $rrds, $expected, "No rrds found" );

# Try with a single host and a single service
$req_params->{host}    = "host1";
$req_params->{service} = "serviceX";
$expected              = {
    hosts    => { host1    => 1 },
    services => { serviceX => 1 },
    metrics  => {
        metricA => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            }
        ]
    },
    errors       => [],
    metric_order => ["metricA"],
    summary      => {
        hosts                => 1,
        services             => 1,
        metrics              => 1,
        host_services        => 1,
        host_service_metrics => 1,
        uoms                 => 1,
    },
    uoms => {
        seconds => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            }
        ]
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Single host/service - all metrics" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = [ "host1", "host2" ];
$req_params->{service} = "serviceX";
$expected              = {
    hosts    => { host1    => 1, host2 => 1 },
    services => { serviceX => 1 },
    metrics  => {
        metricA => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            },
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            }
        ]
    },
    errors       => [],
    metric_order => ["metricA"],
    summary      => {
        hosts                => 2,
        services             => 1,
        metrics              => 1,
        host_services        => 2,
        host_service_metrics => 2,
        uoms                 => 2,
    },
    uoms => {
        "seconds" => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            },
        ],
        "" => [
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            },
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Multiple hosts, single service - all metrics" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = [ "host1",    "host2" ];
$req_params->{service} = [ "serviceX", "serviceY" ];
$expected              = {
    hosts    => { host1    => 1, host2    => 1 },
    services => { serviceX => 1, serviceY => 1 },
    metrics  => {
        metricA => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricA/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceY/metricA/value.rrd",
                host    => "host2",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            }
        ],
        metricB => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            }
        ],
        metricC => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            }
        ],
    },
    errors       => [],
    metric_order => [ "metricA", "metricB", "metricC" ],
    summary      => {
        hosts                => 2,
        services             => 2,
        metrics              => 3,
        host_services        => 4,
        host_service_metrics => 6,
        uoms                 => 2,
    },
    uoms => {
        "seconds" => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            }
        ],
        "" => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricA/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceY/metricA/value.rrd",
                host    => "host2",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            }
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Multiple hosts, multiple services - all metrics" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = [ "host1",    "host2" ];
$req_params->{service} = [ "serviceX", "serviceY" ];
$req_params->{metric}  = "metricA";
$expected              = {
    hosts    => { host1    => 1, host2    => 1 },
    services => { serviceX => 1, serviceY => 1 },
    metrics  => {
        metricA => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricA/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceY/metricA/value.rrd",
                host    => "host2",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            }
        ]
    },
    errors       => [],
    metric_order => ["metricA"],
    summary      => {
        hosts                => 2,
        services             => 2,
        metrics              => 1,
        host_services        => 4,
        host_service_metrics => 4,
        uoms                 => 2,
    },
    uoms => {
        "seconds" => [
            {   file    => "$Bin/var/rrd/host1/serviceX/metricA/value.rrd",
                host    => "host1",
                service => "serviceX",
                name    => "metricA",
                uom     => "seconds",
            }
        ],
        "" => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricA/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceX/metricA/value.rrd",
                host    => "host2",
                service => "serviceX",
                name    => "metricA",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host2/serviceY/metricA/value.rrd",
                host    => "host2",
                service => "serviceY",
                name    => "metricA",
                uom     => "",
            }
        ],
    },

};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Multiple hosts, multiple services - single metric" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = [ "host1",    "host2", "host3" ];
$req_params->{service} = [ "serviceX", "serviceY" ];
$req_params->{metric}  = [ "metricB",  "metricC" ];
$expected              = {
    hosts    => { host1    => 1, host3    => 1, },
    services => { serviceX => 1, serviceY => 1 },
    metrics  => {
        metricB => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceX/metricB/value.rrd",
                host    => "host3",
                service => "serviceX",
                name    => "metricB",
                uom     => "",
            }
        ],
        metricC => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceY/metricC/value.rrd",
                host    => "host3",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            }
        ],
    },
    errors       => [],
    metric_order => [ "metricB", "metricC" ],
    summary      => {
        hosts                => 2,
        services             => 2,
        metrics              => 2,
        host_services        => 3,
        host_service_metrics => 4,
        uoms                 => 1,
    },
    uoms => {
        "" => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceX/metricB/value.rrd",
                host    => "host3",
                service => "serviceX",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceY/metricC/value.rrd",
                host    => "host3",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            }
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Multiple hosts, multiple services - multiple metrics" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = [ "host1",    "host2", "host3" ];
$req_params->{service} = [ "serviceX", "serviceY" ];
$req_params->{metric}  = [ "metricB",  "metricC" ];
$req_params->{includethresholds} = 1;
$expected = {
    hosts    => { host1    => 1, host3    => 1, },
    services => { serviceX => 1, serviceY => 1 },
    metrics  => {
        metricB => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceX/metricB/value.rrd",
                host    => "host3",
                service => "serviceX",
                name    => "metricB",
                uom     => "",
            }
        ],
        metricC => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            },
            {   file      => "$Bin/var/rrd/host3/serviceY/metricC/value.rrd",
                host      => "host3",
                service   => "serviceY",
                name      => "metricC",
                threshold => {
                    file         => "$Bin/var/rrd/host3/serviceY/metricC/thresholds.rrd",
                    warning_end  => 1,
                    critical_end => 1,
                },
                uom => "",
            }
        ],
    },
    errors       => [],
    metric_order => [ "metricB", "metricC" ],
    summary      => {
        hosts                => 2,
        services             => 2,
        metrics              => 2,
        host_services        => 3,
        host_service_metrics => 4,
        uoms                 => 1,
    },
    uoms => {
        "" => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host1/serviceY/metricC/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricC",
                uom     => "",
            },
            {   file    => "$Bin/var/rrd/host3/serviceX/metricB/value.rrd",
                host    => "host3",
                service => "serviceX",
                name    => "metricB",
                uom     => "",
            },
            {   file      => "$Bin/var/rrd/host3/serviceY/metricC/value.rrd",
                host      => "host3",
                service   => "serviceY",
                name      => "metricC",
                threshold => {
                    file         => "$Bin/var/rrd/host3/serviceY/metricC/thresholds.rrd",
                    warning_end  => 1,
                    critical_end => 1,
                },
                uom => "",
            }
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Multiple hosts, multiple services - multiple metrics, including thresholds" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}              = ["host1"];
$req_params->{service}           = [ "serviceX", "serviceY" ];
$req_params->{metric}            = ["metricB"];
$req_params->{includethresholds} = 1;
$public_rrdgraphs                = 0;
$user->mock( "can_view_object_by_name", sub {0} );
$expected = {
    hosts        => {},
    services     => {},
    metrics      => {},
    errors       => [ "messages.accessDeniedToHostService [_1] [_2]:host1:serviceX", "messages.accessDeniedToHostService [_1] [_2]:host1:serviceY", ],
    metric_order => [],
    summary      => {
        hosts                => 0,
        services             => 0,
        metrics              => 0,
        host_services        => 0,
        host_service_metrics => 0,
        uoms                 => 0,
    },
    uoms => {},
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "No public rrdgraphs - error captured" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}              = ["host1"];
$req_params->{service}           = [ "serviceX", "serviceY" ];
$req_params->{metric}            = ["metricB"];
$req_params->{includethresholds} = 1;
$public_rrdgraphs                = 0;
$user->mock(
    "can_view_object_by_name",
    sub {
        my ( $self, $args ) = @_;
        if ( $args->{servicename} eq "serviceY" ) { return 1 }
        return 0;
    }
);
$expected = {
    hosts    => { host1    => 1 },
    services => { serviceY => 1 },
    metrics  => {
        metricB => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            }
        ],
    },
    errors       => [ "messages.accessDeniedToHostService [_1] [_2]:host1:serviceX", ],
    metric_order => ["metricB"],
    summary      => {
        hosts                => 1,
        services             => 1,
        metrics              => 1,
        host_services        => 1,
        host_service_metrics => 1,
        uoms                 => 1,
    },
    uoms => {
        "" => [
            {   file    => "$Bin/var/rrd/host1/serviceY/metricB/value.rrd",
                host    => "host1",
                service => "serviceY",
                name    => "metricB",
                uom     => "",
            }
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Error captured for just serviceX" ) || diag(Data::Dump::dump($rrds));

$req_params->{host}    = ["host3"];
$req_params->{service} = ["service-Z"];
delete $req_params->{metric};
$req_params->{includethresholds} = 0;
$public_rrdgraphs                = 1;
$expected                        = {
    hosts    => { host3       => 1 },
    services => { "service-Z" => 1 },
    metrics  => {
        metricC => [
            {   file    => "$Bin/var/rrd/host3/service%2DZ/metricC/value.rrd",
                host    => "host3",
                service => "service-Z",
                name    => "metricC",
                uom     => "",
            }
        ],
    },
    errors       => [],
    metric_order => ["metricC"],
    summary      => {
        hosts                => 1,
        services             => 1,
        metrics              => 1,
        host_services        => 1,
        host_service_metrics => 1,
        uoms                 => 1,
    },
    uoms => {
        "" => [
            {   file    => "$Bin/var/rrd/host3/service%2DZ/metricC/value.rrd",
                host    => "host3",
                service => "service-Z",
                name    => "metricC",
                uom     => "",
            }
        ],
    },
};
$rrds = Opsview::Web::Controller::Common->list_rrds($c);
is_deeply( $rrds, $expected, "Checking urlencoded-ness" ) || diag(Data::Dump::dump($rrds));

# Check sorting routine
my @input    = qw(load15 load1 load5);
my @expected = qw(load1 load5 load15);
my @result   = Opsview::Web::Controller::Common->_test_sort(@input);
is_deeply( \@result, \@expected, "Sort correct" );
