#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib", "$Bin/lib";

use lib "/usr/local/nagios/perl/lib", "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test;
use Opsview::TestUtils;
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

my $res_data;
my $content;
my $expected;

# Use undef as poor ACLs combined with a saved cookie was allowing access
my $ua = Test::WWW::Mechanize->new( stack_depth => 0, cookie_jar => undef );
$ua->add_header( 'content-type' => 'text/x-data-dumper' );
my $url_prefix = "http://localhost:3000";

$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "admin", password => "initial" }%, "Login to API2" );
is( $ua->status, 200 );
$ua->content_contains( "token", "Should return some ticket information" );
$content = $ua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got expected content" );
my $data = eval $content;
like( $data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$ua->add_header( "X-Opsview-Username" => "admin" );
$ua->add_header( "X-Opsview-Token"    => $data->{token} );

$ua->get_ok("$url_prefix/rest/downtime?rows=0");
is( $ua->status, 200, "Got list of downtime okay" ) || diag explain $ua->content;
$expected = { 'summary' => { 'allrows' => '13', 'rows' => '13', 'num_services' => '9', 'num_hosts' => '4' } };
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got expected summary" ) || diag explain $ua->content;

my $somehosts = Test::WWW::Mechanize->new( stack_depth => 0 );
rest_log_in( $somehosts, "somehosts", "somehosts", $url_prefix );
$somehosts->get_ok("/rest/downtime?rows=0");
$res_data = eval $somehosts->content;
$expected = { 'summary' => { 'allrows' => '6', 'rows' => '6', 'num_services' => '5', 'num_hosts' => '1' } };
is_deeply( $res_data, $expected, "Summary data different from admin user, due to permissions" ) || diag explain $somehosts->content;

$ua->post( "$url_prefix/rest/downtime", Content => '{comment=>"stuff"}' );
is( $ua->status, 403, "Got 403 error code for missing data" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error validating downtime parameters", "Got right error message" );
is( $res_data->{detail}, 'Start time is not defined', "Got right detail" ) || diag( $ua->content );

$ua->post( "$url_prefix/rest/downtime", Content => '{comment=>"stuff", starttime=>"now", endtime=>"tomorrow"}' );
is( $ua->status, 403, "Got 403 error code for no objects chosen" );
$res_data = eval $ua->content;
is( $res_data->{message}, "No objects chosen for downtime creation", "Got right error message" ) || diag( $ua->content );

# Test POSTs for specific objects
# 206=valid service id
$ua->post( "$url_prefix/rest/downtime?hg.hostgroupid=99&hg.hostgroupname=missinghg&hst.hostname=missinghost&hst.hostid=666&svc.hs=badhost::badservice&svc.serviceid=696969&svc.serviceid=206", Content => '{comment=>"autotest", starttime => "2129999900", endtime => "2129999999"}' );
is( $ua->status, 403, "Bad hostgroupid" ) || diag explain $somehosts->content;
$res_data = eval $ua->content;
$expected = {
    message => "Error when searching for objects",
    detail  => "Cannot find host group with name missinghg; Cannot find host group with id 99; Cannot find host with name missinghost; Cannot find host with id 666; Cannot find service with hs badhost::badservice; Cannot find service with serviceid 696969",
};
is_deeply( $res_data, $expected, "Get an error due to hostgroupid which doesn't exist" ) || diag explain $res_data;

# Check using just hostgroupid, hostid and serviceid is sufficient
$ua->post( "$url_prefix/rest/downtime?hg.hostgroupid=99&hst.hostid=6969&svc.serviceid=7777", Content => '{comment=>"autotest", starttime => "2129999900", endtime => "2129999999"}' );
is( $ua->status, 403, "Bad objects" ) || diag explain $somehosts->content;
$res_data = eval $ua->content;
$expected = {
    message => "Error when searching for objects",
    detail  => "Cannot find host group with id 99; Cannot find host with id 6969; Cannot find service with serviceid 7777",
};
is_deeply( $res_data, $expected, "Get an error due to hostgroupid which doesn't exist" ) || diag explain $res_data;

my $cmdfile = "/usr/local/nagios/var/rw/nagios.cmd";
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?hg.hostgroupname=UK", Content => '{comment=>"autotest comment with bad < chars > within ;", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Got ok for hostgroup downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 1,
        num_hosts      => 0,
        num_services   => 0,
    },
    list => {
        hosts      => [],
        hostgroups => [ { name => "UK", id => 3 } ],
        services   => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
my @data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_HOSTGROUP_SVC_DOWNTIME;Leaf;2129999900;2129999999;1;0;;admin;Host group \'UK\': autotest comment with bad  chars  within ', 'SCHEDULE_HOSTGROUP_HOST_DOWNTIME;Leaf;2129999900;2129999999;1;0;;admin;Host group \'UK\': autotest comment with bad  chars  within ', 'SCHEDULE_HOSTGROUP_SVC_DOWNTIME;Monitoring Servers;2129999900;2129999999;1;0;;admin;Host group \'UK\': autotest comment with bad  chars  within ', 'SCHEDULE_HOSTGROUP_HOST_DOWNTIME;Monitoring Servers;2129999900;2129999999;1;0;;admin;Host group \'UK\': autotest comment with bad  chars  within ' ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test somehosts cannot post downtimes
$somehosts->post( "$url_prefix/rest/downtime?hg.hostgroupname=UK", Content => '{comment=>"autotest comment with bad < chars > within ;", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $somehosts->status, 403, "Cannot set downtime for somehosts user" );
$res_data = eval $somehosts->content;
$expected = { message => "No objects chosen for downtime creation", };
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $somehosts->content );

open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?hst.hostname=cisco2&hst.hostname=cloned2&hst.hostid=3", Content => '{comment=>"host downtime", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Got ok for host downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 2,
        num_services   => 0,
    },
    list => {
        hosts      => [ { id => 118, hostname => "cisco2" }, { id => 3, hostname => "cloned2" }, ],
        hostgroups => [],
        services   => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_HOST_SVC_DOWNTIME;cisco2;2129999900;2129999999;1;0;;admin;Host \'cisco2\': host downtime', 'SCHEDULE_HOST_DOWNTIME;cisco2;2129999900;2129999999;1;0;;admin;Host \'cisco2\': host downtime', 'SCHEDULE_HOST_SVC_DOWNTIME;cloned2;2129999900;2129999999;1;0;;admin;Host \'cloned2\': host downtime', 'SCHEDULE_HOST_DOWNTIME;cloned2;2129999900;2129999999;1;0;;admin;Host \'cloned2\': host downtime' ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test somehosts has restricted downtime access
# Also tests duration attribute
open F, ">", $cmdfile;
close F;
$somehosts->post( "$url_prefix/rest/downtime?hst.hostname=cisco2&hst.hostname=cloned2&hst.hostname=monitored_by_slave&hst.hostid=3", Content => '{comment=>"host downtime", starttime=>"2129999900", endtime=>"+5h"}' );
is( $somehosts->status, 403, "Error with restricted objects" );
$res_data = eval $somehosts->content;
$expected = {
    message => "Error when searching for objects",
    detail  => "Cannot find host with name cisco2; Cannot find host with name cloned2; Cannot find host with id 3",
};
is_deeply( $res_data, $expected, "Get an error due to hostgroupid which doesn't exist" ) || diag explain $res_data;

$somehosts->post( "$url_prefix/rest/downtime?hst.hostname=monitored_by_slave&svc.servicegroupname=unix", Content => '{comment=>"host downtime", starttime=>"2129999900", endtime=>"+5h"}' );
is( $somehosts->status, 200, "Got ok for host downtime" );
$res_data = eval $somehosts->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 1,
        num_services   => 1,
    },
    list => {
        hosts      => [ { id => 136, hostname => "monitored_by_slave" }, ],
        hostgroups => [],
        services => [ { id => 193, hostname => "monitored_by_slave", servicename => "/" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $somehosts->content );
open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_HOST_SVC_DOWNTIME;monitored_by_slave;2129999900;2130017900;1;0;;somehosts;Host \'monitored_by_slave\': host downtime', 'SCHEDULE_HOST_DOWNTIME;monitored_by_slave;2129999900;2130017900;1;0;;somehosts;Host \'monitored_by_slave\': host downtime', 'SCHEDULE_SVC_DOWNTIME;monitored_by_slave;/;2129999900;2130017900;1;0;;somehosts;Service \'/\' on host \'monitored_by_slave\': host downtime', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# This also tests that data has priority over URL parameters
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.hostname=cisco2&svc.hostname=%4&svc.servicename=Test exceptions&svc.servicename=/&svc.hostname=opslave&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc downtime\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 3,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ { id => 156, hostname => "cisco2", servicename => "Test exceptions" }, { id => 162, hostname => "cisco4", servicename => "Test exceptions" }, { id => 197, hostname => "opslave", servicename => "/" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_SVC_DOWNTIME;cisco2;Test exceptions;2129999900;2129999999;1;0;;admin;Service \'Test exceptions\' on host \'cisco2\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;cisco4;Test exceptions;2129999900;2129999999;1;0;;admin;Service \'Test exceptions\' on host \'cisco4\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;opslave;/;2129999900;2129999999;1;0;;admin;Service \'/\' on host \'opslave\': svc downtimewith linefeed', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test keyword searching
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.keywordname=cisco_gp2&svc.servicename=Coldstart&starttime=2129999900", Content => "{comment=>'svc downtime\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 3,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ { id => 155, hostname => "cisco2", servicename => "Coldstart" }, { id => 158, hostname => "cisco3", servicename => "Coldstart" }, { id => 161, hostname => "cisco4", servicename => "Coldstart" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_SVC_DOWNTIME;cisco2;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco2\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;cisco3;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco3\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;cisco4;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco4\': svc downtimewith linefeed', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test keyword searching by id
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.keywordid=4&svc.servicename=Coldstart&starttime=2129999900", Content => "{comment=>'svc downtime\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 3,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ { id => 155, hostname => "cisco2", servicename => "Coldstart" }, { id => 158, hostname => "cisco3", servicename => "Coldstart" }, { id => 161, hostname => "cisco4", servicename => "Coldstart" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_SVC_DOWNTIME;cisco2;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco2\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;cisco3;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco3\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;cisco4;Coldstart;2129999900;2129999999;1;0;;admin;Service \'Coldstart\' on host \'cisco4\': svc downtimewith linefeed', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test host group searching - search by id and by hostgroup name at same time
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.servicecheckid=94&svc.hostgroupname=Monitoring+Servers&svc.hostgroupid=2&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc downtime\nwith linefeed', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 2,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ { id => 219, hostname => "opsview", servicename => "Slave-node: opslave" }, { id => 220, hostname => "opsview", servicename => "Slave-node: opslaveclusterA" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_SVC_DOWNTIME;opsview;Slave-node: opslave;2129999900;2129999999;1;0;;admin;Service \'Slave-node: opslave\' on host \'opsview\': svc downtimewith linefeed', 'SCHEDULE_SVC_DOWNTIME;opsview;Slave-node: opslaveclusterA;2129999900;2129999999;1;0;;admin;Service \'Slave-node: opslaveclusterA\' on host \'opsview\': svc downtimewith linefeed', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test somehosts will only be allowed to set downtime for monitored_by_slave::VNC
open F, ">", $cmdfile;
close F;
$somehosts->post( "$url_prefix/rest/downtime?svc.hostname=cisco2&svc.hostname=cisco4&svc.servicename=Test exceptions&svc.hs=monitored_by_slave::VNC&starttime=2129999900&endtime=2129999988", Content => "{comment=>'svc downtime\nwith linefeed', endtime=>'2129999999'}" );
is( $somehosts->status, 200, "Got ok for svc downtime" );
$res_data = eval $somehosts->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 1,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ { id => 5, hostname => "monitored_by_slave", servicename => "VNC" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $somehosts->content );
open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'SCHEDULE_SVC_DOWNTIME;monitored_by_slave;VNC;2129999900;2129999999;1;0;;somehosts;Service \'VNC\' on host \'monitored_by_slave\': svc downtimewith linefeed', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# These deletes will filter by start_time if set
open F, ">", $cmdfile;
close F;
$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/downtime?hst.hostname=cisco2&start_time=1234567890&comment=Bad%20robot") );
is( $ua->status, 200, "Delete downtime succeeded" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 1,
        num_services   => 0,
    },
    list => {
        hosts      => [ { id => 118, hostname => "cisco2" } ],
        hostgroups => [],
        services   => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'DEL_DOWNTIME_BY_HOST_NAME;cisco2;;1234567890;Bad robot', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

open F, ">", $cmdfile;
close F;
$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/downtime?hg.hostgroupname=UK&start_time=1234567890&comment=Bad%20host<gro>up;") );
is( $ua->status, 200, "Delete downtime succeeded" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 1,
        num_hosts      => 0,
        num_services   => 0,
    },
    list => {
        hosts      => [],
        services   => [],
        hostgroups => [ { name => "UK", id => 3 } ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'DEL_DOWNTIME_BY_HOSTGROUP_NAME;Leaf;;;1234567890;Bad hostgroup', 'DEL_DOWNTIME_BY_HOSTGROUP_NAME;Monitoring Servers;;;1234567890;Bad hostgroup', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

# Test somehosts cannot delete downtimes for host groups
$somehosts->request( HTTP::Request::Common::DELETE("$url_prefix/rest/downtime?hg.hostgroupname=UK&start_timev=1234567890&end_timev=1299999999&comment=Bad%20host<gro>up;") );
is( $somehosts->status, 403, "Delete downtime failed for somehosts" );
$res_data = eval $somehosts->content;
$expected = { message => "No objects chosen for downtime deletion", };
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $somehosts->content );

# Service downtime deletes
open F, ">", $cmdfile;
close F;
$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/downtime?svc.hs=opslave::Nagios%20Stats&svc.hs=monitored_by_slave::VNC&start_time=1234567890&comment=Introspection") );
is( $ua->status, 200, "Delete downtime succeeded" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 2,
    },
    list => {
        hostgroups => [],
        hosts      => [],
        services   => [ { id => 222, hostname => "opslave", servicename => "Nagios Stats", }, { id => 5, hostname => "monitored_by_slave", servicename => "VNC" }, ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 'DEL_DOWNTIME_BY_HOST_NAME;opslave;Nagios Stats;1234567890;Introspection', 'DEL_DOWNTIME_BY_HOST_NAME;monitored_by_slave;VNC;1234567890;Introspection', ];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;

note('downtime search tests');

# test to see how '%20' matches - whether treated as a space, or as "<wildcard>20"
# first test - normal wildcards with no ambiguity
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.servicename=% test 20 for%&starttime=2129999900&endtime=2129999988", Content => "{comment=>'Testing servicename search pattern', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for searched svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 1,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [
            {   hostname    => "PassiveSlave",
                id          => 226,
                servicename => "Search test 20 for api wildcards",
            },
        ],
    },
};
is_deeply( $res_data, $expected, "Got searched expected response" ) || diag( dump($res_data) );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ "SCHEDULE_SVC_DOWNTIME;PassiveSlave;Search test 20 for api wildcards;2129999900;2129999999;1;0;;admin;Service 'Search test 20 for api wildcards' on host 'PassiveSlave': Testing servicename search pattern", ];
is_deeply( \@data, $expected, "Got searched commands as expected" ) || diag explain \@data;

# second test - wildcard with ambiguity
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.servicename=% test%20 for%&starttime=2129999900&endtime=2129999988", Content => "{comment=>'Testing servicename search pattern', endtime=>'2129999999'}" );
is( $ua->status, 403, "Got ok for searched svc downtime" );
$res_data = eval $ua->content;
$expected = {
    message => 'No objects chosen for downtime creation',
};
is_deeply( $res_data, $expected, "Got searched expected response" ) || diag( dump($res_data) );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ ];
is_deeply( \@data, $expected, "Got searched commands as expected" ) || diag explain \@data;

# third test - wildcard % uri-escaped to %25 in the request
open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?svc.servicename=% test%2520 for%&starttime=2129999900&endtime=2129999988", Content => "{comment=>'Testing servicename search pattern', endtime=>'2129999999'}" );
is( $ua->status, 200, "Got ok for searched svc downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 1,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [
            {   hostname    => "PassiveSlave",
                id          => 226,
                servicename => "Search test 20 for api wildcards",
            },
        ],
    },
};
is_deeply( $res_data, $expected, "Got searched expected response" ) || diag( dump($res_data) );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ "SCHEDULE_SVC_DOWNTIME;PassiveSlave;Search test 20 for api wildcards;2129999900;2129999999;1;0;;admin;Service 'Search test 20 for api wildcards' on host 'PassiveSlave': Testing servicename search pattern", ];
is_deeply( \@data, $expected, "Got searched commands as expected" ) || diag explain \@data;



# Check that a get gives the same objects that a POST sets
my $parameters = "hostgroupid=3&hostname=cisco2&hostname=opslave&servicename=/&servicename=Coldstart";
$ua->get_ok("$url_prefix/rest/status/service?$parameters");
is( $ua->status, 200, "Got filtered downtime okay" );
$res_data = eval $ua->content;
is( $res_data->{list}->[0]->{name}, "cisco2", "Got hostname cisco2" );
is( $res_data->{list}->[0]->{services}->[0]->{name}, "Coldstart", "With service Coldstart" );
is( $res_data->{list}->[1]->{name}, "opslave", ".. and opslave" );
is( $res_data->{list}->[1]->{services}->[0]->{name}, "/", "With service /" );

open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?$parameters", Content => '{comment=>"autotest comment with bad < chars > within ;", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Got ok for hostgroup downtime" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 0,
        num_services   => 2,
    },
    list => {
        hosts      => [],
        hostgroups => [],
        services   => [ 
            { id => 155, hostname => "cisco2", servicename => "Coldstart" }, 
            { id => 197, hostname => "opslave", servicename => "/" }, 
        ],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 
    "SCHEDULE_SVC_DOWNTIME;cisco2;Coldstart;2129999900;2129999999;1;0;;admin;Service 'Coldstart' on host 'cisco2': autotest comment with bad  chars  within ",
    "SCHEDULE_SVC_DOWNTIME;opslave;/;2129999900;2129999999;1;0;;admin;Service '/' on host 'opslave': autotest comment with bad  chars  within ",
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


open F, ">", $cmdfile;
close F;
$ua->post( "$url_prefix/rest/downtime?host=cisco4&host=cisco3", Content => '{comment=>"autotest comment with bad < chars > within ;", starttime=>"2129999900", endtime=>"2129999999"}' );
is( $ua->status, 200, "Checking that host object is returned rather than a list of services as downtime uses smart_object_search=>1" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        num_hostgroups => 0,
        num_hosts      => 2,
        num_services   => 0,
    },
    list => {
        hosts      => [
            { id => 119, hostname => "cisco3" },
            { id => 120, hostname => "cisco4" },
        ],
        hostgroups => [],
        services   => [],
    },
};
is_deeply( $res_data, $expected, "Got expected response" ) || diag( $ua->content );

open F, $cmdfile;
@data = map { s/^\[\d+\] //; chomp; $_ } <F>;
close F;
$expected = [ 
    "SCHEDULE_HOST_SVC_DOWNTIME;cisco3;2129999900;2129999999;1;0;;admin;Host 'cisco3': autotest comment with bad  chars  within ",
    "SCHEDULE_HOST_DOWNTIME;cisco3;2129999900;2129999999;1;0;;admin;Host 'cisco3': autotest comment with bad  chars  within ",
    "SCHEDULE_HOST_SVC_DOWNTIME;cisco4;2129999900;2129999999;1;0;;admin;Host 'cisco4': autotest comment with bad  chars  within ",
    "SCHEDULE_HOST_DOWNTIME;cisco4;2129999900;2129999999;1;0;;admin;Host 'cisco4': autotest comment with bad  chars  within ",
];
is_deeply( \@data, $expected, "Got commands as expected" ) || diag explain \@data;


